/**
 * @author David
 */

// global variables
var statement = '';

// Updates the current logic statement with the user's characters
// and allows us to construct the statement manually.
function addToStatement(op, update) {
  var text = document.getElementById("logicalstatement");
  switch(op) {
    // parenthesis
    case '(':
      text.innerHTML += ' ( ';
      if (update) {
        statement += '(';
      }
      break;
    case ')':
      text.innerHTML += ' ) ';
      if (update) {
        statement += ')';
      }
      break;
      
    // logical operators
    case 'a':
      text.innerHTML += '&and;';
      if (update) {
        statement += 'a';
      }
      break;
    case 'o':
      text.innerHTML += '&or;';
      if (update) {
        statement += 'o';
      }
      break;
    case '!':
      text.innerHTML += '&not;';
      if (update) {
        statement += '!';
      }
      break;
    case 'x':
      text.innerHTML += '&#8891;';

      // XOR bitwise operator
      if (update) {
        statement += 'x';
      }
      break;
    case 'n':
      text.innerHTML += '&#8893;';

      // needs to be translated later
      if (update) {
        statement += 'n';
      }
      break;
    case 'c':
      text.innerHTML += '&rArr;';

      // needs to be translated later
      if (update) {
        statement += 'c';
      }
      break;
    case 'b':
      text.innerHTML += '&hArr;';

      // needs to be translated later
      if (update) {
        statement += 'b';
      }
      break;
      
    // Propositions
    case 'P':
      text.innerHTML += 'P';
      if (update) {
        statement += 'P';
      }
      break;
    case 'Q':
      text.innerHTML += 'Q';
      if (update) {
        statement += 'Q';
      }
      break;
    case 'R':
      text.innerHTML += 'R';
      if (update) {
        statement += 'R';
      }
      break;
  }
}

// Create the truth table from the statement
function createTruthTable() {
  // Since we construct the string manually from known characters, there is no reason
  // to filter it.
  
  // create a table to hold our results
  var table = new Array();
  
  // try all combinations of P, Q, and R being true or false, using bitwise operations
  for (var i = 0; i < 2; i++) {
    
    table[i] = new Array();
    for (var j = 0; j < 2; j++) {
      
      table[i][j] = new Array();
      for (var k = 0; k < 2; k++) {
      
        // replace the P, Q, and R with their bit equivalents
        var s = statement.replaceAll("P", i).replaceAll("Q", j).replaceAll("R", k);
        
        try {
          // we check all combinations, even ones where we don't need the results
          table[i][j][k] = eval(findTruthValue(s, true));
        }
        catch(e) {
          debug("Error in making table");
          return;
        }
      }
    }
  }

  // now draw the table
  // make sure to eliminate P, Q or R columns if they are not present in the statement
  drawTable(table);
}

/*
 * Recursively search through the statement converting each simple block found
 * into a bit equivalent ( 0 or 1 ) depending on the truth value of the block
 * Note: We have already replaced as much of the statement as we can with actual
 * JavaScript code so we can eval() it later.
 * 
 * @Parameters
 * t - A string to find the truth value of
 *
 * @Returns
 * Either true or false, depending on the truth value of the block
 *
 */
function findTruthValue(t) {
  // loop through replacing each section with the appropriate rules
  // 1. check for matching blocks like 0&1 etc...
  // 2. replace the block with its appropriate truth value 
  // 3. replace (0) or (1) with 0 and 1
  // 4. repeat until the string is either a 1 or a 0
  
  // for now let's stop at 50 iterations, just in case
  var count = 0;
  
  // temporay matching variable
  var temp;
  
  // regular expression matcher
  var logic = new RegExp("[01][abcnox][01]");
  
  while (count < 500 && (t != 0 || t != 1)) {
    // 1. replace !0 or !1 first
    t = t.replace("!0", "1").replace("!1", "0");
  
    // 2. check for matching blocks with 3 characters each
    if (temp = logic.exec(t)) {
      var m = temp[0];
      operator = m[1];
      switch (operator) {
        // easy connectives
        case 'a':
          // just evaluate them now and replace
          t = t.replace(m, eval(m.replace("a", "&")));
          break;
        case 'o':
          // just evaluate them now and replace
          t = t.replace(m, eval(m.replace("o", "|")));
          break;
        case 'x':
          // just evaluate them now and replace
          t = t.replace(m, eval(m.replace("x", "^")));
          break;

        // here we substitute the correct value, simplifying as we go
        // NOR operator (P NOR Q == !P & !Q)
        case 'n':
          t = t.replace(m, eval("!" + m[0] + "&!" + m[2]));
          break;
        
        // conditional (P -> Q == !P | Q)
        case 'c':
          t = t.replace(m, eval("!" + m[0] + "|" + m[2]));
          break;
      
        // biconditional (P <-> Q == ((!P | Q) & (!Q | P))
        case 'b':
          t = t.replace(m, eval("(!" + m[0] + "|" + m[2] + ")&(!" + m[2] + "|" + m[0] + ")"));
          break;
      }
    }
    
    // 3. replace (0) and (1) with 0 and 1
    t = t.replace("(0)", "0").replace("(1)", "1");
    
    //debug(t);
    count++;
  }
  return t;
}

/*
 * Create an HTML table from a given table of truth values
 *
 * @Parameter
 * table - a 3-dimensional array of truth values
 *
 * @Return
 * none
 */
function drawTable(table) {

  // first check to see if we need P, Q and R in our truth table
  var needP = /P/.test(statement);
  var needQ = /Q/.test(statement);
  var needR = /R/.test(statement);

  if (!needP && !needQ && !needR) {
    return;
  }

  var output = "<table class='truth'><tr>";
  output += (needP ? "<th>P</th>" : "") + (needQ ? "<th>Q</th>" : "") + (needR ? "<th>R</th>" : "") + "<th><span class='cell-statement'>" + document.getElementById("logicalstatement").innerHTML + "</span></th>";
  output += "</tr>";

  // first flatten our array
  var tmp = new Object();
  
  for (var i = 0; i < 2; i++) {
    for (var j = 0; j < 2; j++) {
      for (var k = 0; k < 2; k++) {
        tmp["row-" + needP*i + needQ*j + needR*k] = (needP ? '<td class="' + cellClass(i) + '">' + toTF(i) + '</td>' : '');
        tmp["row-" + needP*i + needQ*j + needR*k] += (needQ ? '<td class="' + cellClass(j) + '">' + toTF(j) + '</td>' : '');
        tmp["row-" + needP*i + needQ*j + needR*k] += (needR ? '<td class="' + cellClass(k) + '">' + toTF(k) + '</td>' : '');
        tmp["row-" + needP*i + needQ*j + needR*k] += '<td class="' + cellClass(table[needP*i][needQ*j][needR*k]) + '">' + toTF(table[needP*i][needQ*j][needR*k]) + '</td>';
      }
    }
  }
  
  // each entry in the table is a row in our table
  for (row in tmp) {
    output += "<tr>" + tmp[row] + "</tr>";  
  }
  
  output += "</table>";
  debug(output, true);
}

// Convert 1 and 0 to true and false respectively
function toTF(number) {
  return (number == 0) ? "false" : "true";
}

// Convert 1 and 0 to cell-true and cell-false respectively
function cellClass(number) {
  return (number == 0) ? "cell-false" : "cell-true";
}

// Clear the previous logical statements
function clearStatement() {
  document.getElementById("logicalstatement").innerHTML = "";
  document.getElementById("truthtable").innerHTML = "";
  statement = '';
}

// Undo the previous action
function undoStatement() {
  statement = statement.substr(0, statement.length - 1);
  document.getElementById("logicalstatement").innerHTML = "";
  var n = statement.length;
  for (var i = 0; i < n; i++) {
    addToStatement(statement[i], false);
  }
}

// //debug function
function debug(s, replaceContent) {
  if (replaceContent) {
    document.getElementById("truthtable").innerHTML = s + "<br />";
  }
  else {
    document.getElementById("truthtable").innerHTML += s + "<br />";
  }
}

/*
 * Replaces all instances of the given substring.
 * Borrowed from http://www.bennadel.com
 */
String.prototype.replaceAll = function(strTarget,strSubString){
  var strText = this;
  var intIndexOfMatch = strText.indexOf( strTarget );
    
  // Keep looping while an instance of the target string
  // still exists in the string.
  while (intIndexOfMatch != -1){
    // Relace out the current instance.
    strText = strText.replace( strTarget, strSubString )
     
    // Get the index of any next matching substring.
    intIndexOfMatch = strText.indexOf( strTarget );
  }
    
  // Return the updated string with ALL the target strings
  // replaced out with the new substring.
  return (strText);
}